#include "t6963c.h"
#include <util/delay.h>
#include <avr/interrupt.h>

//-----------------------------------------------------------------------------------------
// NASTAVENI DISPLEJE
//-----------------------------------------------------------------------------------------

// datovy port
#define LCD_DATA_DDR	DDRD
#define LCD_DATA_PORT 	PORTD
#define LCD_DATA_PIN	PIND

//	ridici port
#define LCD_CTRL_DDR	DDRB
#define LCD_CTRL_PORT	PORTB
#define LCD_CTRL_PIN	PINB

// ridici signaly
#define LCD_WR			PINB0
#define LCD_RD			PINB1
#define LCD_CE			PINB2
#define LCD_CD			PINB3
#define LCD_RST			PINB4
#define LCD_FS			PINB5

// nastaveni pameti displeje
#define LCD_X_PIXELS						128
#define LCD_Y_PIXELS						64
#define LCD_FONT_WIDTH						8

#define LCD_GRAPHIC_AREA					(LCD_X_PIXELS / LCD_FONT_WIDTH)
#define LCD_TEXT_AREA						(LCD_X_PIXELS / LCD_FONT_WIDTH)
#define LCD_GRAPHIC_SIZE					(LCD_GRAPHIC_AREA * LCD_Y_PIXELS)
#define LCD_TEXT_SIZE						(LCD_TEXT_AREA * (LCD_Y_PIXELS/8))

#define LCD_TEXT_HOME						0
#define LCD_GRAPHIC_HOME					(LCD_TEXT_HOME + LCD_TEXT_SIZE)
#define LCD_OFFSET_REGISTER					2
#define LCD_EXTERNAL_CG_HOME				(LCD_OFFSET_REGISTER << 11)


//-----------------------------------------------------------------------------------------
// Cteni stavu displeje
//-----------------------------------------------------------------------------------------
unsigned char DISPLAY_StatusCheck()
{
	uint8_t status;
	LCD_DATA_DDR = 0x00;

	LCD_CTRL_PORT &= ~((1 << LCD_RD) | (1 << LCD_CE)); // read status
	_delay_us(1.5);
	status = LCD_DATA_PIN;
	
	LCD_DATA_DDR = 0xff;
	LCD_CTRL_PORT |= ((1 << LCD_RD) | (1 << LCD_CE));
	
	return status;
}


//-----------------------------------------------------------------------------------------
// Zapsani instrukce
//-----------------------------------------------------------------------------------------
void DISPLAY_WriteCommand(unsigned char command)
{
	while(!(DISPLAY_StatusCheck()&0x03)); // kontrola stavu STA0 a STA1
	LCD_DATA_PORT = command; // zapis prikazu
	
	// zapis instrukce
	LCD_CTRL_PORT &= ~((1 << LCD_WR) | (1 << LCD_CE));
	LCD_CTRL_PORT |= ((1 << LCD_WR) | (1 << LCD_CE)); 
}


//-----------------------------------------------------------------------------------------
// Zapsani prikazu v AW modu
//-----------------------------------------------------------------------------------------
void DISPLAY_WriteCommandAw(unsigned char command)
{
	while(!(DISPLAY_StatusCheck()&0x08)); // kontrola stavu STA3
	LCD_DATA_PORT = command; //zapis prikazu
	_delay_us(1.5);

	LCD_CTRL_PORT &= ~((1 << LCD_WR) | (1 << LCD_CE));
	LCD_CTRL_PORT |= ((1 << LCD_WR) | (1 << LCD_CE));
}


//-----------------------------------------------------------------------------------------
// Zapsani dat
//-----------------------------------------------------------------------------------------
void DISPLAY_WriteData(unsigned char data)
{
	while(!(DISPLAY_StatusCheck()&0x03)); // kontrola stavu STA0 a STA1
	LCD_DATA_PORT = data; // zapis dat

	LCD_CTRL_PORT &= ~((1 << LCD_WR) | (1 << LCD_CE) | (1 << LCD_CD));
	LCD_CTRL_PORT |= ((1 << LCD_WR) | (1 << LCD_CE) | (1 << LCD_CD));
}


//-----------------------------------------------------------------------------------------
// Zapsani dat v AW modu
//-----------------------------------------------------------------------------------------
void DISPLAY_WriteDataAw(unsigned char data)
{
	while(!(DISPLAY_StatusCheck()&0x08)); // kontrola stavu STA3
	LCD_DATA_PORT = data; // zapis dat
	_delay_us(1.5);

	LCD_CTRL_PORT &= ~((1 << LCD_WR) | (1 << LCD_CE) | (1 << LCD_CD));
	LCD_CTRL_PORT |= ((1 << LCD_WR) | (1 << LCD_CE) | (1 << LCD_CD));
}


//-----------------------------------------------------------------------------------------
// Cteni displeje
//-----------------------------------------------------------------------------------------
unsigned char DISPLAY_ReadData()
{
	uint8_t tmp;
	while(!(DISPLAY_StatusCheck()&0x03));
	LCD_DATA_DDR = 0x00;

	LCD_CTRL_PORT &= ~((1 << LCD_RD) | (1 << LCD_CE) | (1 << LCD_CD));
	_delay_us(1.5);
	tmp = LCD_DATA_PIN;
	LCD_CTRL_PORT |= ((1 << LCD_RD) | (1 << LCD_CE) | (1 << LCD_CD));
	LCD_DATA_DDR = 0xff;
	return tmp;
}


//-----------------------------------------------------------------------------------------
// Nastaveni ukazatele pameti
//------------------------------------------------------------------------------------------
void DISPLAY_SetAddressPointer(unsigned int address)
{
	DISPLAY_WriteData(address & 0xff);
	DISPLAY_WriteData(address >> 8);
	DISPLAY_WriteCommand(T6963c_SET_ADDRESS_POINTER);
}


//-----------------------------------------------------------------------------------------
// Zapnuti modu Auto Write
//-----------------------------------------------------------------------------------------
void DISPLAY_AwOn()
{
	DISPLAY_WriteCommand(T6963c_SET_DATA_AUTO_WRITE);
}


//-----------------------------------------------------------------------------------------
// Vypnuti modu Auto Write
//-----------------------------------------------------------------------------------------
void DISPLAY_AwOff()
{
	DISPLAY_WriteCommandAw(T6963c_AUTO_RESET);
}


//-----------------------------------------------------------------------------------------
// Vymazani pameti displeje
//-----------------------------------------------------------------------------------------
void DISPLAY_ClearMemory()
{
	unsigned int i;
	
	// mazani CG pameti v rezimu AW
	DISPLAY_SetAddressPointer(LCD_EXTERNAL_CG_HOME);

	DISPLAY_AwOn();
	for(i = 0; i < 256 * 8; i++)
	{
		DISPLAY_WriteDataAw(0x00);
		DISPLAY_WriteCommandAw(T6963c_DATA_WRITE_AND_INCREMENT);
	}
	DISPLAY_AwOff();

	// mazani graficke pameti v rezimu AW
	DISPLAY_SetAddressPointer(LCD_GRAPHIC_HOME);
	DISPLAY_AwOn();
	for(i = 0; i < LCD_GRAPHIC_SIZE; i++)
	{
		DISPLAY_WriteDataAw(0x00);
		DISPLAY_WriteCommandAw(T6963c_DATA_WRITE_AND_INCREMENT);
	}
	DISPLAY_AwOff();
	
	// mazani textove pameti v rezimu AW
	DISPLAY_SetAddressPointer(LCD_TEXT_HOME);
	DISPLAY_AwOn();
	for(i = 0; i < LCD_TEXT_SIZE; i++)
	{
		DISPLAY_WriteDataAw(0x00);
		DISPLAY_WriteCommandAw(T6963c_DATA_WRITE_AND_INCREMENT);
	}
	DISPLAY_AwOff();
}


//-----------------------------------------------------------------------------------------
// Inicializace displeje
//-----------------------------------------------------------------------------------------
void DISPLAY_Init()
{
	// reset displeje
	LCD_DATA_DDR = 0xff;
	LCD_CTRL_PORT &= ~(1 << LCD_RST);
	_delay_us(2);
	LCD_CTRL_PORT |= (1 << LCD_RST);
	
	// inicializace portu
	LCD_CTRL_DDR = ((1 << LCD_WR) | (1 << LCD_RD) | (1 << LCD_CE) | (1 << LCD_CD) | (1 << LCD_RST) | (1 << LCD_FS));
	LCD_CTRL_PORT |= ((1 << LCD_WR) | (1 << LCD_RD) | (1 << LCD_CE) | (1 << LCD_CD) | (1 << LCD_RST) | (1 << LCD_FS));

	// nastaveni sirky fontu
	if (LCD_FONT_WIDTH == 8)
	{
		LCD_CTRL_PORT &= ~(1 << LCD_FS);
	}

	// nastaveni pocatku graficke adresy
	DISPLAY_WriteData(LCD_GRAPHIC_HOME & 0xff);
	DISPLAY_WriteData(LCD_GRAPHIC_HOME >> 8);
	DISPLAY_WriteCommand(T6963c_SET_GRAPHIC_HOME_ADDRESS);

	//nastaveni rozsahu graficke pameti
	DISPLAY_WriteData(LCD_GRAPHIC_AREA);
	DISPLAY_WriteData(0x00);
	DISPLAY_WriteCommand(T6963c_SET_GRAPHIC_AREA);

	//nastaveni pocatku textove pameti
	DISPLAY_WriteData(LCD_TEXT_HOME);
	DISPLAY_WriteData(LCD_TEXT_HOME >> 8);
	DISPLAY_WriteCommand(T6963c_SET_TEXT_HOME_ADDRESS);

	// nastaveni rozsahu textovee pameti
	DISPLAY_WriteData(LCD_TEXT_AREA);
	DISPLAY_WriteData(0x00);
	DISPLAY_WriteCommand(T6963c_SET_TEXT_AREA);
	
	// prolinani textove a graficke casti OR
    DISPLAY_WriteCommand(T6963c_OR_MODE);
	
	//vymazani pameti
	DISPLAY_ClearMemory();
	
	//zapnuti grafickeho rezimu
	DISPLAY_WriteCommand(T6963c_GRAPHIC_DISPLAY_ON);
		
}


//-----------------------------------------------------------------------------------------
// Funkce PIXEL
//-----------------------------------------------------------------------------------------
void DISPLAY_SetPixel(short int x, short int y, unsigned char color)
{
	unsigned char data;
	unsigned int address = LCD_GRAPHIC_HOME + (x / LCD_FONT_WIDTH) + (LCD_GRAPHIC_AREA * y);

	DISPLAY_SetAddressPointer(address);

	DISPLAY_WriteCommand(T6963c_DATA_READ_AND_NONVARIABLE);
	data = DISPLAY_ReadData();

	if(color)
	data |= (1 <<  (LCD_FONT_WIDTH - 1 - (x % LCD_FONT_WIDTH)));
	else
	data &= ~(1 <<  (LCD_FONT_WIDTH - 1 - (x % LCD_FONT_WIDTH)));

	DISPLAY_WriteData(data);
	DISPLAY_WriteCommand(T6963c_DATA_WRITE_AND_INCREMENT);
}